@@ -101,6 +101,7 @@ panel_launcher_save_ditem (GnomeDesktopItem *ditem,
 		panel_error_dialog (screen,
 				    "cannot_save_launcher",
 				    _("Cannot save launcher to disk"),
+				    "%s",
 				    error->message);
 
 		g_error_free (error);
@@ -137,8 +138,9 @@ launch_url (Launcher *launcher)
 		panel_error_dialog (screen,
 				    "cannot_show_url_dialog",
 				    _("Cannot show %s"),
-				    error->message,
-				    url);
+				    "%s",
+				    url,
+				    error->message);
 
 		g_clear_error (&error);
 	}
@@ -170,6 +172,7 @@ launch_cb (GtkWidget *widget,
 			panel_error_dialog (launcher_get_screen (launcher),
 					    "cannot_launch_icon",
 					    _("Cannot launch icon"),
+					    "%s",
 					    error->message);
 
 			g_clear_error (&error);
@@ -215,6 +218,7 @@ drag_data_received_cb (GtkWidget        *widget,
 		panel_error_dialog (launcher_get_screen (launcher),
 				    "cannot_launch_icon",
 				    _("Cannot launch icon"),
+				    "%s",
 				    error->message);
 		g_clear_error (&error);
 	}
@@ -510,7 +514,7 @@ setup_button (Launcher *launcher)
 	/* Setup icon */
 	icon = gnome_desktop_item_get_string (launcher->ditem,
 					      GNOME_DESKTOP_ITEM_ICON);
-	button_widget_set_icon_name (BUTTON_WIDGET (launcher->button), icon);
+	button_widget_set_icon_name (BUTTON_WIDGET (launcher->button), sure_string (icon));
 
 	/* Setup help */
 	docpath = gnome_desktop_item_get_string (launcher->ditem,
@@ -620,10 +624,14 @@ window_response (GtkWidget *w, int response, gpointer data)
 		 * signal again
 		 */
 		g_signal_connect (G_OBJECT (launcher->dedit), "changed",
-				    G_CALLBACK (launcher_changed),
-				    launcher);
+				  G_CALLBACK (launcher_changed),
+				  launcher);
 
-		properties_apply (launcher);
+		/* resave launcher */
+		launcher_save (launcher);
+
+		/* Setup the button look */
+		setup_button (launcher);
 
 	} else {
 		gtk_widget_destroy (w);
@@ -932,31 +940,6 @@ ask_about_launcher (const char  *file,
 	gnome_ditem_edit_grab_focus (dee);
 }
 
-static void
-ditem_set_icon (GnomeDesktopItem *ditem, const char *icon)
-{
-	if (icon != NULL &&
-	    icon[0] != G_DIR_SEPARATOR) {
-		char *full = gnome_desktop_item_find_icon (panel_icon_theme,
-							   icon,
-							   48 /* desired size */,
-							   0 /* flags */);
-		if (full != NULL) {
-			gnome_desktop_item_set_string (ditem,
-						       GNOME_DESKTOP_ITEM_ICON,
-						       full);
-			g_free (full);
-		} else {
-			gnome_desktop_item_set_string (ditem,
-						       GNOME_DESKTOP_ITEM_ICON,
-						       icon);
-		}
-	} else {
-		gnome_desktop_item_set_string (ditem, GNOME_DESKTOP_ITEM_ICON,
-					       icon);
-	}
-}
-
 void
 panel_launcher_create_from_info (PanelToplevel *toplevel,
 				 int            position,
@@ -978,7 +961,7 @@ panel_launcher_create_from_info (PanelToplevel *toplevel,
 	gnome_desktop_item_set_localestring (ditem, GNOME_DESKTOP_ITEM_NAME, name);
 	gnome_desktop_item_set_localestring (ditem, GNOME_DESKTOP_ITEM_COMMENT, comment);
 
-	ditem_set_icon (ditem, icon);
+	gnome_desktop_item_set_string (ditem, GNOME_DESKTOP_ITEM_ICON, icon);
 
 	if (exec_info) {
 		gnome_desktop_item_set_string (ditem, GNOME_DESKTOP_ITEM_EXEC, exec_or_uri);
@@ -997,9 +980,9 @@ panel_launcher_create_from_info (PanelToplevel *toplevel,
 }
 
 void
-panel_launcher_create (PanelToplevel *toplevel,
-		       int            position,
-		       const char    *location)
+panel_launcher_create_with_id (const char    *toplevel_id,
+			       int            position,
+			       const char    *location)
 {
 	GConfClient *client;
 	const char  *profile;
@@ -1011,14 +994,23 @@ panel_launcher_create (PanelToplevel *toplevel,
 	client  = panel_gconf_get_client ();
 	profile = panel_profile_get_name ();
 
-	id = panel_profile_prepare_object (PANEL_OBJECT_LAUNCHER, toplevel, position, FALSE);
+	id = panel_profile_prepare_object_with_id (PANEL_OBJECT_LAUNCHER, toplevel_id, position, FALSE);
 
 	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, profile, id, "launcher_location");
 	gconf_client_set_string (client, key, location, NULL);
 
 	/* frees id */
 	panel_profile_add_to_list (PANEL_GCONF_OBJECTS, id);
+}
 
+void
+panel_launcher_create (PanelToplevel *toplevel,
+		       int            position,
+		       const char    *location)
+{
+	panel_launcher_create_with_id (panel_profile_get_toplevel_id (toplevel),
+				       position,
+				       location);
 }
 
 void
@@ -1107,6 +1099,7 @@ launcher_show_help (Launcher  *launcher,
 		panel_error_dialog (screen,
 				    "cannot_show_gnome_kde_help",
 				    _("Cannot display help document"),
+				    "%s",
 				    error->message);
 
 		g_error_free (error);
